@@ -16,10 +16,12 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, see <https://www.gnu.org/licenses/>.
+ * SPDX-License-Identifier: GPL-3.0-or-later
  */
 
 #include <config.h>
 
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <stddef.h>
@@ -29,12 +31,8 @@
 # include <signal.h>
 #endif
 #include <errno.h>
-#include <assert.h>
 #include <sys/time.h>
 #include <unistd.h>
-#ifndef USE_LDAPWRAPPER
-# include <npth.h>
-#endif
 
 #ifdef HAVE_W32_SYSTEM
 # include <winsock2.h>
@@ -51,7 +49,6 @@
 
 #include <gpg-error.h>
 #include "../common/logging.h"
-#include "../common/argparse.h"
 #include "../common/stringhelp.h"
 #include "../common/mischelp.h"
 #include "../common/strlist.h"
@@ -60,43 +57,12 @@
 #include "../common/util.h"
 #include "../common/init.h"
 
-/* With the ldap wrapper, there is no need for the npth_unprotect and leave
-   functions; thus we redefine them to nops.  If we are not using the
-   ldap wrapper process we need to include the prototype for our
-   module's main function.  */
-#ifdef USE_LDAPWRAPPER
+/* There is no need for the npth_unprotect and leave functions here;
+ * thus we redefine them to nops.  We keep them in the code just for
+ * the case we ever want to reuse parts of the code in npth programs. */
 static void npth_unprotect (void) { }
 static void npth_protect (void) { }
-#else
-# include "./ldap-wrapper.h"
-#endif
 
-#ifdef HAVE_W32CE_SYSTEM
-# include "w32-ldap-help.h"
-# define my_ldap_init(a,b)                      \
-  _dirmngr_ldap_init ((a), (b))
-# define my_ldap_simple_bind_s(a,b,c)           \
-  _dirmngr_ldap_simple_bind_s ((a),(b),(c))
-# define my_ldap_search_st(a,b,c,d,e,f,g,h)     \
-  _dirmngr_ldap_search_st ((a), (b), (c), (d), (e), (f), (g), (h))
-# define my_ldap_first_attribute(a,b,c)         \
-  _dirmngr_ldap_first_attribute ((a),(b),(c))
-# define my_ldap_next_attribute(a,b,c)          \
-  _dirmngr_ldap_next_attribute ((a),(b),(c))
-# define my_ldap_get_values_len(a,b,c)          \
-  _dirmngr_ldap_get_values_len ((a),(b),(c))
-# define my_ldap_free_attr(a)                   \
-  xfree ((a))
-#else
-# define my_ldap_init(a,b)              ldap_init ((a), (b))
-# define my_ldap_simple_bind_s(a,b,c)   ldap_simple_bind_s ((a), (b), (c))
-# define my_ldap_search_st(a,b,c,d,e,f,g,h)     \
-  ldap_search_st ((a), (b), (c), (d), (e), (f), (g), (h))
-# define my_ldap_first_attribute(a,b,c) ldap_first_attribute ((a),(b),(c))
-# define my_ldap_next_attribute(a,b,c)  ldap_next_attribute ((a),(b),(c))
-# define my_ldap_get_values_len(a,b,c)  ldap_get_values_len ((a),(b),(c))
-# define my_ldap_free_attr(a)           ldap_memfree ((a))
-#endif
 
 #ifdef HAVE_W32_SYSTEM
  typedef LDAP_TIMEVAL  my_ldap_timeval_t;
@@ -104,7 +70,7 @@ static void npth_protect (void) { }
  typedef struct timeval my_ldap_timeval_t;
 #endif
 
-#define DEFAULT_LDAP_TIMEOUT 100 /* Arbitrary long timeout. */
+#define DEFAULT_LDAP_TIMEOUT 15 /* Arbitrary long timeout. */
 
 
 /* Constants for the options.  */
@@ -124,6 +90,7 @@ enum
     oDN,
     oFilter,
     oAttr,
+    oTls,
 
     oOnlySearchTimeout,
     oLogWithPID
@@ -131,7 +98,7 @@ enum
 
 
 /* The list of options as used by the argparse.c code.  */
-static ARGPARSE_OPTS opts[] = {
+static gpgrt_opt_t opts[] = {
   { oVerbose,  "verbose",   0, N_("verbose") },
   { oQuiet,    "quiet",     0, N_("be somewhat more quiet") },
   { oTimeout,  "timeout",   1, N_("|N|set LDAP timeout to N seconds")},
@@ -139,6 +106,7 @@ static ARGPARSE_OPTS opts[] = {
                                   " a record oriented format")},
   { oProxy,    "proxy",     2,
     N_("|NAME|ignore host part and connect through NAME")},
+  { oTls,      "tls",       0, N_("force a TLS connection")},
   { oHost,     "host",      2, N_("|NAME|connect to host NAME")},
   { oPort,     "port",      1, N_("|N|connect to port N")},
   { oUser,     "user",      2, N_("|NAME|use user NAME for authentication")},
@@ -150,7 +118,7 @@ static ARGPARSE_OPTS opts[] = {
   { oAttr,     "attr",      2, N_("|STRING|return the attribute STRING")},
   { oOnlySearchTimeout, "only-search-timeout", 0, "@"},
   { oLogWithPID,"log-with-pid", 0, "@"},
-  { 0, NULL, 0, NULL }
+  ARGPARSE_end ()
 };
 
 
@@ -164,6 +132,7 @@ struct my_opt_s
   my_ldap_timeval_t timeout;/* Timeout for the LDAP search functions.  */
   unsigned int alarm_timeout; /* And for the alarm based timeout.  */
   int multi;
+  int force_tls;
 
   estream_t outstream;    /* Send output to this stream.  */
 
@@ -190,17 +159,18 @@ static int process_url (my_opt_t myopt, const char *url);
 
 
 /* Function called by argparse.c to display information.  */
-#ifdef USE_LDAPWRAPPER
 static const char *
 my_strusage (int level)
 {
   const char *p;
 
-  switch(level)
+  switch (level)
     {
+    case  9: p = "GPL-3.0-or-later"; break;
     case 11: p = "dirmngr_ldap (@GNUPG@)";
       break;
     case 13: p = VERSION; break;
+    case 14: p = GNUPG_DEF_COPYRIGHT_LINE; break;
     case 17: p = PRINTABLE_OS_NAME; break;
     case 19: p = _("Please report bugs to <@EMAIL@>.\n"); break;
     case 49: p = PACKAGE_BUGREPORT; break;
@@ -218,20 +188,12 @@ my_strusage (int level)
     }
   return p;
 }
-#endif /*!USE_LDAPWRAPPER*/
 
 
 int
-#ifdef USE_LDAPWRAPPER
 main (int argc, char **argv)
-#else
-ldap_wrapper_main (char **argv, estream_t outstream)
-#endif
 {
-#ifndef USE_LDAPWRAPPER
-  int argc;
-#endif
-  ARGPARSE_ARGS pargs;
+  gpgrt_argparse_t pargs;
   int any_err = 0;
   char *p;
   int only_search_timeout = 0;
@@ -243,8 +205,7 @@ ldap_wrapper_main (char **argv, estream_t outstream)
 
   early_system_init ();
 
-#ifdef USE_LDAPWRAPPER
-  set_strusage (my_strusage);
+  gpgrt_set_strusage (my_strusage);
   log_set_prefix ("dirmngr_ldap", GPGRT_LOG_WITH_PREFIX);
 
   /* Setup I18N and common subsystems. */
@@ -254,11 +215,6 @@ ldap_wrapper_main (char **argv, estream_t outstream)
 
   es_set_binary (es_stdout);
   myopt->outstream = es_stdout;
-#else /*!USE_LDAPWRAPPER*/
-  myopt->outstream = outstream;
-  for (argc=0; argv[argc]; argc++)
-    ;
-#endif /*!USE_LDAPWRAPPER*/
 
   /* LDAP defaults */
   myopt->timeout.tv_sec = DEFAULT_LDAP_TIMEOUT;
@@ -268,8 +224,8 @@ ldap_wrapper_main (char **argv, estream_t outstream)
   /* Parse the command line.  */
   pargs.argc = &argc;
   pargs.argv = &argv;
-  pargs.flags= 1;  /* Do not remove the args. */
-  while (arg_parse (&pargs, opts) )
+  pargs.flags= ARGPARSE_FLAG_KEEP;
+  while (gpgrt_argparse (NULL, &pargs, opts))
     {
       switch (pargs.r_opt)
         {
@@ -288,6 +244,7 @@ ldap_wrapper_main (char **argv, estream_t outstream)
           myopt->pass = getenv ("DIRMNGR_LDAP_PASS");
           break;
         case oProxy: myopt->proxy = pargs.r.ret_str; break;
+        case oTls:  myopt->force_tls = 1; break;
         case oHost: myopt->host = pargs.r.ret_str; break;
         case oPort: myopt->port = pargs.r.ret_int; break;
         case oDN:   myopt->dn = pargs.r.ret_str; break;
@@ -302,14 +259,11 @@ ldap_wrapper_main (char **argv, estream_t outstream)
           break;
 
         default :
-#ifdef USE_LDAPWRAPPER
           pargs.err = ARGPARSE_PRINT_ERROR;
-#else
-          pargs.err = ARGPARSE_PRINT_WARNING;  /* No exit() please.  */
-#endif
           break;
 	}
     }
+  gpgrt_argparse (NULL, &pargs, NULL);
 
   if (only_search_timeout)
     myopt->alarm_timeout = 0;
@@ -336,17 +290,11 @@ ldap_wrapper_main (char **argv, estream_t outstream)
   if (myopt->port < 0 || myopt->port > 65535)
     log_error (_("invalid port number %d\n"), myopt->port);
 
-#ifdef USE_LDAPWRAPPER
   if (log_get_errorcount (0))
     exit (2);
   if (argc < 1)
-    usage (1);
-#else
-  /* All passed arguments should be fine in this case.  */
-  assert (argc);
-#endif
+    gpgrt_usage (1);
 
-#ifdef USE_LDAPWRAPPER
   if (myopt->alarm_timeout)
     {
 #ifndef HAVE_W32_SYSTEM
@@ -363,7 +311,6 @@ ldap_wrapper_main (char **argv, estream_t outstream)
           log_fatal ("unable to register timeout handler\n");
 #endif
     }
-#endif /*USE_LDAPWRAPPER*/
 
   for (; argc; argc--, argv++)
     if (process_url (myopt, *argv))
@@ -382,16 +329,56 @@ catch_alarm (int dummy)
 }
 #endif
 
+
+#ifdef HAVE_W32_SYSTEM
+static DWORD CALLBACK
+alarm_thread (void *arg)
+{
+  HANDLE timer = arg;
+
+  WaitForSingleObject (timer, INFINITE);
+  _exit (10);
+
+  return 0;
+}
+#endif
+
+
 static void
 set_timeout (my_opt_t myopt)
 {
-#ifdef HAVE_W32_SYSTEM
-  /* FIXME for W32.  */
-  (void)myopt;
-#else
   if (myopt->alarm_timeout)
-    alarm (myopt->alarm_timeout);
+    {
+#ifdef HAVE_W32_SYSTEM
+      static HANDLE timer;
+      LARGE_INTEGER due_time;
+
+      /* A negative value is a relative time.  */
+      due_time.QuadPart = (unsigned long long)-10000000 * myopt->alarm_timeout;
+
+      if (!timer)
+        {
+          SECURITY_ATTRIBUTES sec_attr;
+          DWORD tid;
+
+          memset (&sec_attr, 0, sizeof sec_attr);
+          sec_attr.nLength = sizeof sec_attr;
+          sec_attr.bInheritHandle = FALSE;
+
+          /* Create a manual resettable timer.  */
+          timer = CreateWaitableTimer (NULL, TRUE, NULL);
+          /* Initially set the timer.  */
+          SetWaitableTimer (timer, &due_time, 0, NULL, NULL, 0);
+
+          if (CreateThread (&sec_attr, 0, alarm_thread, timer, 0, &tid))
+            log_error ("failed to create alarm thread\n");
+        }
+      else /* Retrigger the timer.  */
+        SetWaitableTimer (timer, &due_time, 0, NULL, NULL, 0);
+#else
+      alarm (myopt->alarm_timeout);
 #endif
+    }
 }
 
 
@@ -424,10 +411,10 @@ print_ldap_entries (my_opt_t myopt, LDAP *ld, LDAPMessage *msg, char *want_attr)
         }
 
 
-      for (npth_unprotect (), attr = my_ldap_first_attribute (ld, item, &berctx),
+      for (npth_unprotect (), attr = ldap_first_attribute (ld, item, &berctx),
              npth_protect ();
            attr;
-           npth_unprotect (), attr = my_ldap_next_attribute (ld, item, berctx),
+           npth_unprotect (), attr = ldap_next_attribute (ld, item, berctx),
              npth_protect ())
         {
           struct berval **values;
@@ -460,20 +447,20 @@ print_ldap_entries (my_opt_t myopt, LDAP *ld, LDAPMessage *msg, char *want_attr)
                 *cp2 = ';';
               if (cmpres)
                 {
-                  my_ldap_free_attr (attr);
+                  ldap_memfree (attr);
                   continue; /* Not found:  Try next attribute.  */
                 }
             }
 
           npth_unprotect ();
-          values = my_ldap_get_values_len (ld, item, attr);
+          values = ldap_get_values_len (ld, item, attr);
           npth_protect ();
 
           if (!values)
             {
               if (myopt->verbose)
                 log_info (_("attribute '%s' not found\n"), attr);
-              my_ldap_free_attr (attr);
+              ldap_memfree (attr);
               continue;
             }
 
@@ -503,7 +490,7 @@ print_ldap_entries (my_opt_t myopt, LDAP *ld, LDAPMessage *msg, char *want_attr)
                   log_error (_("error writing to stdout: %s\n"),
                              strerror (errno));
                   ldap_value_free_len (values);
-                  my_ldap_free_attr (attr);
+                  ldap_memfree (attr);
                   ber_free (berctx, 0);
                   return -1;
                 }
@@ -527,7 +514,7 @@ print_ldap_entries (my_opt_t myopt, LDAP *ld, LDAPMessage *msg, char *want_attr)
                       log_error (_("error writing to stdout: %s\n"),
                                  strerror (errno));
                       ldap_value_free_len (values);
-                      my_ldap_free_attr (attr);
+                      ldap_memfree (attr);
                       ber_free (berctx, 0);
                       return -1;
                     }
@@ -539,7 +526,7 @@ print_ldap_entries (my_opt_t myopt, LDAP *ld, LDAPMessage *msg, char *want_attr)
                   log_error (_("error writing to stdout: %s\n"),
                              strerror (errno));
                   ldap_value_free_len (values);
-                  my_ldap_free_attr (attr);
+                  ldap_memfree (attr);
                   ber_free (berctx, 0);
                   return -1;
                 }
@@ -549,7 +536,7 @@ print_ldap_entries (my_opt_t myopt, LDAP *ld, LDAPMessage *msg, char *want_attr)
                 break; /* Print only the first value.  */
             }
           ldap_value_free_len (values);
-          my_ldap_free_attr (attr);
+          ldap_memfree (attr);
           if (want_attr || !myopt->multi)
             break; /* We only want to return the first attribute.  */
         }
@@ -574,6 +561,7 @@ fetch_ldap (my_opt_t myopt, const char *url, const LDAPURLDesc *ludp)
   char *host, *dn, *filter, *attrs[2], *attr;
   int port;
   int ret;
+  int usetls;
 
   host     = myopt->host?   myopt->host   : ludp->lud_host;
   port     = myopt->port?   myopt->port   : ludp->lud_port;
@@ -583,12 +571,19 @@ fetch_ldap (my_opt_t myopt, const char *url, const LDAPURLDesc *ludp)
   attrs[1] = NULL;
   attr = attrs[0];
 
-  if (!port)
+  if (!port && myopt->force_tls)
+    port = 636;
+  else if (!port)
     port = (ludp->lud_scheme && !strcmp (ludp->lud_scheme, "ldaps"))? 636:389;
 
   if (myopt->verbose)
     {
       log_info (_("processing url '%s'\n"), url);
+      if (myopt->force_tls)
+        log_info ("forcing tls\n");
+      else
+        log_info ("not forcing tls\n");
+
       if (myopt->user)
         log_info (_("          user '%s'\n"), myopt->user);
       if (myopt->pass)
@@ -626,20 +621,70 @@ fetch_ldap (my_opt_t myopt, const char *url, const LDAPURLDesc *ludp)
       && ludp->lud_attrs && ludp->lud_attrs[0] && ludp->lud_attrs[1])
     log_info (_("WARNING: using first attribute only\n"));
 
-
   set_timeout (myopt);
-  npth_unprotect ();
-  ld = my_ldap_init (host, port);
-  npth_protect ();
-  if (!ld)
+
+  usetls = (myopt->force_tls
+            || (ludp->lud_scheme && !strcmp (ludp->lud_scheme, "ldaps")));
+#if HAVE_W32_SYSTEM
+  if (1)
     {
-      log_error (_("LDAP init to '%s:%d' failed: %s\n"),
-                 host, port, strerror (errno));
-      return -1;
+      npth_unprotect ();
+      ld = ldap_sslinit (host, port, usetls);
+      npth_protect ();
+      if (!ld)
+        {
+          ret = LdapGetLastError ();
+          log_error (_("LDAP init to '%s:%d' failed: %s\n"),
+                     host, port, ldap_err2string (ret));
+          return -1;
+        }
     }
+#else /*!W32*/
+  if (usetls)
+    {
+      char *uri;
+
+      uri = xtryasprintf ("ldaps://%s:%d", host, port);
+      if (!uri)
+        {
+          log_error (_("error allocating memory: %s\n"),
+                     gpg_strerror (gpg_error_from_syserror ()));
+          return -1;
+        }
+      npth_unprotect ();
+      ret = ldap_initialize (&ld, uri);
+      npth_protect ();
+      if (ret)
+        {
+          log_error (_("LDAP init to '%s' failed: %s\n"),
+                     uri, ldap_err2string (ret));
+          xfree (uri);
+          return -1;
+        }
+      else if (myopt->verbose)
+        log_info (_("LDAP init to '%s' done\n"), uri);
+      xfree (uri);
+    }
+  else
+    {
+      /* Keep the old way so to avoid regressions.  Eventually we
+       * should really consider the supplied scheme and use only
+       * ldap_initialize.  */
+      npth_unprotect ();
+      ld = ldap_init (host, port);
+      npth_protect ();
+      if (!ld)
+        {
+          log_error (_("LDAP init to '%s:%d' failed: %s\n"),
+                     host, port, strerror (errno));
+          return -1;
+        }
+    }
+#endif /*!W32*/
+
   npth_unprotect ();
   /* Fixme:  Can we use MYOPT->user or is it shared with other theeads?.  */
-  ret = my_ldap_simple_bind_s (ld, myopt->user, myopt->pass);
+  ret = ldap_simple_bind_s (ld, myopt->user, myopt->pass);
   npth_protect ();
 #ifdef LDAP_VERSION3
   if (ret == LDAP_PROTOCOL_ERROR)
@@ -650,7 +695,7 @@ fetch_ldap (my_opt_t myopt, const char *url, const LDAPURLDesc *ludp)
         log_info ("protocol error; retrying bind with v3 protocol\n");
       npth_unprotect ();
       ldap_set_option (ld, LDAP_OPT_PROTOCOL_VERSION, &version);
-      ret = my_ldap_simple_bind_s (ld, myopt->user, myopt->pass);
+      ret = ldap_simple_bind_s (ld, myopt->user, myopt->pass);
       npth_protect ();
     }
 #endif
@@ -664,11 +709,11 @@ fetch_ldap (my_opt_t myopt, const char *url, const LDAPURLDesc *ludp)
 
   set_timeout (myopt);
   npth_unprotect ();
-  rc = my_ldap_search_st (ld, dn, ludp->lud_scope, filter,
-                          myopt->multi && !myopt->attr && ludp->lud_attrs?
-                          ludp->lud_attrs:attrs,
-                          0,
-                          &myopt->timeout, &msg);
+  rc = ldap_search_st (ld, dn, ludp->lud_scope, filter,
+                       myopt->multi && !myopt->attr && ludp->lud_attrs?
+                       ludp->lud_attrs:attrs,
+                       0,
+                       &myopt->timeout, &msg);
   npth_protect ();
   if (rc == LDAP_SIZELIMIT_EXCEEDED && myopt->multi)
     {
@@ -680,12 +725,8 @@ fetch_ldap (my_opt_t myopt, const char *url, const LDAPURLDesc *ludp)
     }
   else if (rc)
     {
-#ifdef HAVE_W32CE_SYSTEM
-      log_error ("searching '%s' failed: %d\n", url, rc);
-#else
       log_error (_("searching '%s' failed: %s\n"),
                  url, ldap_err2string (rc));
-#endif
       if (rc != LDAP_NO_SUCH_OBJECT)
         {
           /* FIXME: Need deinit (ld)?  */
