@@ -144,40 +144,9 @@ gpg_dirmngr_deinit_session_data (ctrl_t ctrl)
 static gpg_error_t
 warn_version_mismatch (assuan_context_t ctx, const char *servername)
 {
-  gpg_error_t err;
-  char *serverversion;
-  const char *myversion = strusage (13);
-
-  err = get_assuan_server_version (ctx, 0, &serverversion);
-  if (err)
-    log_error (_("error getting version from '%s': %s\n"),
-               servername, gpg_strerror (err));
-  else if (compare_version_strings (serverversion, myversion) < 0)
-    {
-      char *warn;
-
-      warn = xtryasprintf (_("server '%s' is older than us (%s < %s)"),
-                           servername, serverversion, myversion);
-      if (!warn)
-        err = gpg_error_from_syserror ();
-      else
-        {
-          log_info (_("WARNING: %s\n"), warn);
-          if (!opt.quiet)
-            {
-              log_info (_("Note: Outdated servers may lack important"
-                          " security fixes.\n"));
-              log_info (_("Note: Use the command \"%s\" to restart them.\n"),
-                        "gpgconf --kill all");
-            }
-
-          write_status_strings (STATUS_WARNING, "server_version_mismatch 0",
-                                " ", warn, NULL);
-          xfree (warn);
-        }
-    }
-  xfree (serverversion);
-  return err;
+  return warn_server_version_mismatch (ctx, servername, 0,
+                                       write_status_strings2, NULL,
+                                       !opt.quiet);
 }
 
 
@@ -394,7 +363,8 @@ ks_status_cb (void *opaque, const char *line)
   struct ks_status_parm_s *parm = opaque;
   gpg_error_t err = 0;
   const char *s, *s2;
-  const char *warn;
+  const char *warn = NULL;
+  int is_note = 0;
 
   if ((s = has_leading_keyword (line, parm->keyword? parm->keyword : "SOURCE")))
     {
@@ -406,18 +376,35 @@ ks_status_cb (void *opaque, const char *line)
             err = gpg_error_from_syserror ();
         }
     }
-  else if ((s = has_leading_keyword (line, "WARNING")))
+  else if ((s = has_leading_keyword (line, "WARNING"))
+           || (is_note = !!(s = has_leading_keyword (line, "NOTE"))))
     {
-      if ((s2 = has_leading_keyword (s, "tor_not_running")))
+      if ((s2 = has_leading_keyword (s, "wkd_cached_result")))
+        {
+          if (opt.verbose)
+            warn = _("WKD uses a cached result");
+        }
+      else if ((s2 = has_leading_keyword (s, "tor_not_running")))
         warn = _("Tor is not running");
       else if ((s2 = has_leading_keyword (s, "tor_config_problem")))
         warn = _("Tor is not properly configured");
+      else if ((s2 = has_leading_keyword (s, "dns_config_problem")))
+        warn = _("DNS is not properly configured");
+      else if ((s2 = has_leading_keyword (s, "http_redirect")))
+        warn = _("unacceptable HTTP redirect from server");
+      else if ((s2 = has_leading_keyword (s, "http_redirect_cleanup")))
+        warn = _("unacceptable HTTP redirect from server was cleaned up");
+      else if ((s2 = has_leading_keyword (s, "tls_cert_error")))
+        warn = _("server uses an invalid certificate");
       else
         warn = NULL;
 
       if (warn)
         {
-          log_info (_("WARNING: %s\n"), warn);
+          if (is_note)
+            log_info (_("Note: %s\n"), warn);
+          else
+            log_info (_("WARNING: %s\n"), warn);
           if (s2)
             {
               while (*s2 && !spacep (s2))
@@ -944,6 +931,7 @@ ks_put_inq_cb (void *opaque, const char *line)
     {
       kbnode_t node;
       estream_t fp;
+      char hexfpr[2*MAX_FINGERPRINT_LEN+1];
 
       /* Parse the keyblock and send info lines back to the server.  */
       fp = es_fopenmem (0, "rw,samethread");
@@ -1001,6 +989,8 @@ ks_put_inq_cb (void *opaque, const char *line)
 			       nbits_from_pk (pk), pk->pubkey_algo,
 			       pk->keyid, pk->timestamp, pk->expiredate,
 			       NULL);
+                es_fprintf (fp, "fpr:::::::::%s:\n",
+                            hexfingerprint (pk, hexfpr, sizeof hexfpr));
               }
               break;
 
@@ -1028,21 +1018,6 @@ ks_put_inq_cb (void *opaque, const char *line)
               }
               break;
 
-              /* This bit is really for the benefit of people who
-                 store their keys in LDAP servers.  It makes it easy
-                 to do queries for things like "all keys signed by
-                 Isabella".  */
-            case PKT_SIGNATURE:
-              {
-                PKT_signature *sig = node->pkt->pkt.signature;
-
-                if (IS_UID_SIG (sig))
-		  record_output (fp, node->pkt->pkttype, NULL,
-				 -1, -1, sig->keyid,
-				 sig->timestamp, sig->expiredate, NULL);
-              }
-              break;
-
             default:
               continue;
             }
@@ -1081,7 +1056,7 @@ ks_put_inq_cb (void *opaque, const char *line)
 
 /* Send a key to the configured server.  {DATA,DATLEN} contains the
    key in OpenPGP binary transport format.  If KEYBLOCK is not NULL it
-   has the internal representaion of that key; this is for example
+   has the internal representation of that key; this is for example
    used to convey meta data to LDAP keyservers.  */
 gpg_error_t
 gpg_dirmngr_ks_put (ctrl_t ctrl, void *data, size_t datalen, kbnode_t keyblock)
@@ -1274,72 +1249,6 @@ gpg_dirmngr_dns_cert (ctrl_t ctrl, const char *name, const char *certtype,
 }
 
 
-/* Ask the dirmngr for PKA info.  On success the retrieved fingerprint
-   is returned in a malloced buffer at R_FPR and its length is stored
-   at R_FPRLEN.  If an URL is available it is stored as a malloced
-   string at R_URL.  On error all return values are set to NULL/0.  */
-gpg_error_t
-gpg_dirmngr_get_pka (ctrl_t ctrl, const char *userid,
-                     unsigned char **r_fpr, size_t *r_fprlen,
-                     char **r_url)
-{
-  gpg_error_t err;
-  assuan_context_t ctx;
-  struct dns_cert_parm_s parm;
-  char *line = NULL;
-
-  memset (&parm, 0, sizeof parm);
-  if (r_fpr)
-    *r_fpr = NULL;
-  if (r_fprlen)
-    *r_fprlen = 0;
-  if (r_url)
-    *r_url = NULL;
-
-  err = open_context (ctrl, &ctx);
-  if (err)
-    return err;
-
-  line = es_bsprintf ("DNS_CERT --pka -- %s", userid);
-  if (!line)
-    {
-      err = gpg_error_from_syserror ();
-      goto leave;
-    }
-  if (strlen (line) + 2 >= ASSUAN_LINELENGTH)
-    {
-      err = gpg_error (GPG_ERR_TOO_LARGE);
-      goto leave;
-    }
-
-  err = assuan_transact (ctx, line, dns_cert_data_cb, &parm,
-                         NULL, NULL, dns_cert_status_cb, &parm);
-  if (err)
-    goto leave;
-
-  if (r_fpr && parm.fpr)
-    {
-      *r_fpr = parm.fpr;
-      parm.fpr = NULL;
-    }
-  if (r_fprlen)
-    *r_fprlen = parm.fprlen;
-
-  if (r_url && parm.url)
-    {
-      *r_url = parm.url;
-      parm.url = NULL;
-    }
-
- leave:
-  xfree (parm.fpr);
-  xfree (parm.url);
-  xfree (line);
-  close_context (ctrl, ctx);
-  return err;
-}
-
-
 
 /* Ask the dirmngr to retrieve a key via the Web Key Directory
  * protocol.  If QUICK is set the dirmngr is advised to use a shorter
