@@ -42,6 +42,7 @@
 #include "trustdb.h"
 #include "call-agent.h"
 #include "key-clean.h"
+#include "pkglue.h"
 
 
 /* An object to keep track of subkeys. */
@@ -92,10 +93,9 @@ static int do_export_stream (ctrl_t ctrl, iobuf_t out,
                              strlist_t users, int secret,
                              kbnode_t *keyblock_out, unsigned int options,
 			     export_stats_t stats, int *any);
-static gpg_error_t print_pka_or_dane_records
+static gpg_error_t print_dane_records
 /**/                 (iobuf_t out, kbnode_t keyblock, PKT_public_key *pk,
-                      const void *data, size_t datalen,
-                      int print_pka, int print_dane);
+                      const void *data, size_t datalen);
 
 
 static void
@@ -108,7 +108,7 @@ cleanup_export_globals (void)
 }
 
 
-/* Option parser for export options.  See parse_options fro
+/* Option parser for export options.  See parse_options for
    details.  */
 int
 parse_export_options(char *str,unsigned int *options,int noisy)
@@ -126,7 +126,6 @@ parse_export_options(char *str,unsigned int *options,int noisy)
       {"export-minimal",EXPORT_MINIMAL|EXPORT_CLEAN,NULL,
        N_("remove as much as possible from key during export")},
 
-      {"export-pka", EXPORT_PKA_FORMAT, NULL, NULL },
       {"export-dane", EXPORT_DANE_FORMAT, NULL, NULL },
 
       {"backup", EXPORT_BACKUP, NULL,
@@ -147,14 +146,18 @@ parse_export_options(char *str,unsigned int *options,int noisy)
   int rc;
 
   rc = parse_options (str, options, export_opts, noisy);
-  if (rc && (*options & EXPORT_BACKUP))
+  if (!rc)
+    return 0;
+
+  /* Alter other options we want or don't want for restore.  */
+  if ((*options & EXPORT_BACKUP))
     {
-      /* Alter other options we want or don't want for restore.  */
       *options |= (EXPORT_LOCAL_SIGS | EXPORT_ATTRIBUTES
                    | EXPORT_SENSITIVE_REVKEYS);
       *options &= ~(EXPORT_CLEAN | EXPORT_MINIMAL
-                    | EXPORT_PKA_FORMAT | EXPORT_DANE_FORMAT);
+                    | EXPORT_DANE_FORMAT);
     }
+
   return rc;
 }
 
@@ -408,7 +411,7 @@ do_export (ctrl_t ctrl, strlist_t users, int secret, unsigned int options,
   if (rc)
     return rc;
 
-  if ( opt.armor && !(options & (EXPORT_PKA_FORMAT|EXPORT_DANE_FORMAT)) )
+  if ( opt.armor && !(options & EXPORT_DANE_FORMAT) )
     {
       afx = new_armor_context ();
       afx->what = secret? 5 : 1;
@@ -493,10 +496,8 @@ exact_subkey_match_p (KEYDB_SEARCH_DESC *desc, kbnode_t node)
       keyid_from_pk (node->pkt->pkt.public_key, kid);
       break;
 
-    case KEYDB_SEARCH_MODE_FPR16:
-    case KEYDB_SEARCH_MODE_FPR20:
     case KEYDB_SEARCH_MODE_FPR:
-      fingerprint_from_pk (node->pkt->pkt.public_key, fpr,&fprlen);
+      fingerprint_from_pk (node->pkt->pkt.public_key, fpr, &fprlen);
       break;
 
     default:
@@ -515,14 +516,8 @@ exact_subkey_match_p (KEYDB_SEARCH_DESC *desc, kbnode_t node)
         result = 1;
       break;
 
-    case KEYDB_SEARCH_MODE_FPR16:
-      if (!memcmp (desc->u.fpr, fpr, 16))
-        result = 1;
-      break;
-
-    case KEYDB_SEARCH_MODE_FPR20:
     case KEYDB_SEARCH_MODE_FPR:
-      if (!memcmp (desc->u.fpr, fpr, 20))
+      if (fprlen == desc->fprlen && !memcmp (desc->u.fpr, fpr, desc->fprlen))
         result = 1;
       break;
 
@@ -567,7 +562,9 @@ match_curve_skey_pk (gcry_sexp_t s_key, PKT_public_key *pk)
       log_error ("no curve name\n");
       return gpg_error (GPG_ERR_UNKNOWN_CURVE);
     }
-  oidstr = openpgp_curve_to_oid (curve_str, NULL);
+  if (!strcmp (curve_str, "Ed448"))
+    is_eddsa = 1;
+  oidstr = openpgp_curve_to_oid (curve_str, NULL, NULL);
   if (!oidstr)
     {
       log_error ("no OID known for curve '%s'\n", curve_str);
@@ -606,7 +603,7 @@ match_curve_skey_pk (gcry_sexp_t s_key, PKT_public_key *pk)
 }
 
 
-/* Return a canonicalized public key algoithms.  This is used to
+/* Return a canonicalized public key algorithms.  This is used to
    compare different flavors of algorithms (e.g. ELG and ELG_E are
    considered the same).  */
 static enum gcry_pk_algos
@@ -750,10 +747,8 @@ cleartext_secret_key_to_openpgp (gcry_sexp_t s_key, PKT_public_key *pk)
       err = match_curve_skey_pk (key, pk);
       if (err)
         goto leave;
-      if (!err)
-        err = gcry_sexp_extract_param (key, NULL, "q",
-                                       &pub_params[0],
-                                       NULL);
+      else
+        err = sexp_extract_param_sos (key, "q", &pub_params[0]);
       if (!err && (gcry_mpi_cmp(pk->pkey[1], pub_params[0])))
         err = gpg_error (GPG_ERR_BAD_PUBKEY);
 
@@ -764,9 +759,7 @@ cleartext_secret_key_to_openpgp (gcry_sexp_t s_key, PKT_public_key *pk)
         {
           gcry_mpi_release (pk->pkey[sec_start]);
           pk->pkey[sec_start] = NULL;
-          err = gcry_sexp_extract_param (key, NULL, "d",
-                                         &pk->pkey[sec_start],
-                                         NULL);
+          err = sexp_extract_param_sos (key, "d", &pk->pkey[sec_start]);
         }
 
       if (!err)
@@ -978,15 +971,31 @@ transfer_format_to_openpgp (gcry_sexp_t s_pgp, PKT_public_key *pk)
       value = gcry_sexp_nth_data (list, ++idx, &valuelen);
       if (!value || !valuelen)
         goto bad_seckey;
-      if (is_enc)
+      if (is_enc
+          || pk->pubkey_algo == PUBKEY_ALGO_ECDSA
+          || pk->pubkey_algo == PUBKEY_ALGO_EDDSA
+          || pk->pubkey_algo == PUBKEY_ALGO_ECDH)
         {
-          void *p = xtrymalloc (valuelen);
-          if (!p)
-            goto outofmem;
-          memcpy (p, value, valuelen);
-          skey[skeyidx] = gcry_mpi_set_opaque (NULL, p, valuelen*8);
+          unsigned int nbits = valuelen*8;
+          const unsigned char *p = value;
+
+          if (*p && nbits >= 8 && !(*p & 0x80))
+            if (--nbits >= 7 && !(*p & 0x40))
+              if (--nbits >= 6 && !(*p & 0x20))
+                if (--nbits >= 5 && !(*p & 0x10))
+                  if (--nbits >= 4 && !(*p & 0x08))
+                    if (--nbits >= 3 && !(*p & 0x04))
+                      if (--nbits >= 2 && !(*p & 0x02))
+                        if (--nbits >= 1 && !(*p & 0x01))
+                          --nbits;
+
+          skey[skeyidx] = gcry_mpi_set_opaque_copy (NULL, value, nbits);
           if (!skey[skeyidx])
             goto outofmem;
+          if (is_enc)
+            gcry_mpi_set_flag (skey[skeyidx], GCRYMPI_FLAG_USER1);
+          else
+            gcry_mpi_set_flag (skey[skeyidx], GCRYMPI_FLAG_USER2);
         }
       else
         {
@@ -1028,11 +1037,11 @@ transfer_format_to_openpgp (gcry_sexp_t s_pgp, PKT_public_key *pk)
   /* log_debug ("XXX pubkey_algo=%d\n", pubkey_algo); */
   /* log_debug ("XXX is_protected=%d\n", is_protected); */
   /* log_debug ("XXX protect_algo=%d\n", protect_algo); */
-  /* log_printhex (iv, ivlen, "XXX iv"); */
+  /* log_printhex ("XXX iv", iv, ivlen); */
   /* log_debug ("XXX ivlen=%d\n", ivlen); */
   /* log_debug ("XXX s2k_mode=%d\n", s2k_mode); */
   /* log_debug ("XXX s2k_algo=%d\n", s2k_algo); */
-  /* log_printhex (s2k_salt, sizeof s2k_salt, "XXX s2k_salt"); */
+  /* log_printhex ("XXX s2k_salt", s2k_salt, sizeof s2k_salt); */
   /* log_debug ("XXX s2k_count=%lu\n", (unsigned long)s2k_count); */
   /* for (idx=0; skey[idx]; idx++) */
   /*   { */
@@ -1043,7 +1052,7 @@ transfer_format_to_openpgp (gcry_sexp_t s_pgp, PKT_public_key *pk)
   /*         void *p; */
   /*         unsigned int nbits; */
   /*         p = gcry_mpi_get_opaque (skey[idx], &nbits); */
-  /*         log_printhex ( p, (nbits+7)/8, NULL); */
+  /*         log_printhex (NULL, p, (nbits+7)/8); */
   /*       } */
   /*     else */
   /*       gcry_mpi_dump (skey[idx]); */
@@ -1073,7 +1082,7 @@ transfer_format_to_openpgp (gcry_sexp_t s_pgp, PKT_public_key *pk)
           goto leave;
         }
 
-      oidstr = openpgp_curve_to_oid (curve, NULL);
+      oidstr = openpgp_curve_to_oid (curve, NULL, NULL);
       if (!oidstr)
         {
           log_error ("no OID known for curve '%s'\n", curve);
@@ -1110,7 +1119,7 @@ transfer_format_to_openpgp (gcry_sexp_t s_pgp, PKT_public_key *pk)
       /*         void *p; */
       /*         unsigned int nbits; */
       /*         p = gcry_mpi_get_opaque (skey[idx], &nbits); */
-      /*         log_printhex (p, (nbits+7)/8, NULL); */
+      /*         log_printhex (NULL, p, (nbits+7)/8); */
       /*       } */
       /*     else */
       /*       gcry_mpi_dump (skey[idx]); */
@@ -1144,7 +1153,7 @@ transfer_format_to_openpgp (gcry_sexp_t s_pgp, PKT_public_key *pk)
   /* Check that the first secret key parameter in SKEY is encrypted
      and that there are no more secret key parameters.  The latter is
      guaranteed by the v4 packet format.  */
-  if (!gcry_mpi_get_flag (skey[npkey], GCRYMPI_FLAG_OPAQUE))
+  if (!gcry_mpi_get_flag (skey[npkey], GCRYMPI_FLAG_USER1))
     goto bad_seckey;
   if (npkey+1 < DIM (skey) && skey[npkey+1])
     goto bad_seckey;
@@ -1224,7 +1233,7 @@ print_status_exported (PKT_public_key *pk)
  * passphrase-protected.  Otherwise, store secret key material in the
  * clear.
  *
- * CACHE_NONCE_ADDR is used to share nonce for multple key retrievals.
+ * CACHE_NONCE_ADDR is used to share nonce for multiple key retrievals.
  */
 gpg_error_t
 receive_seckey_from_agent (ctrl_t ctrl, gcry_cipher_hd_t cipherhd,
@@ -1323,7 +1332,7 @@ write_keyblock_to_output (kbnode_t keyblock, int with_armor,
   if (opt.verbose)
     log_info (_("writing to '%s'\n"), iobuf_get_fname_nonnull (out));
 
-  if ((options & (EXPORT_PKA_FORMAT|EXPORT_DANE_FORMAT)))
+  if ((options & EXPORT_DANE_FORMAT))
     {
       with_armor = 0;
       out_help = iobuf_temp ();
@@ -1360,7 +1369,7 @@ write_keyblock_to_output (kbnode_t keyblock, int with_armor,
     }
   err = 0;
 
-  if (out_help && pk)
+  if (out_help && pk && (options & EXPORT_DANE_FORMAT))
     {
       const void *data;
       size_t datalen;
@@ -1369,10 +1378,7 @@ write_keyblock_to_output (kbnode_t keyblock, int with_armor,
       data = iobuf_get_temp_buffer (out_help);
       datalen = iobuf_get_temp_length (out_help);
 
-      err = print_pka_or_dane_records (out,
-                                       keyblock, pk, data, datalen,
-                                       (options & EXPORT_PKA_FORMAT),
-                                       (options & EXPORT_DANE_FORMAT));
+      err = print_dane_records (out, keyblock, pk, data, datalen);
     }
 
  leave:
@@ -1463,13 +1469,12 @@ apply_drop_subkey_filter (ctrl_t ctrl, kbnode_t keyblock,
 }
 
 
-/* Print DANE or PKA records for all user IDs in KEYBLOCK to OUT.  The
- * data for the record is taken from (DATA,DATELEN).  PK is the public
- * key packet with the primary key. */
+/* Print DANErecords for all user IDs in KEYBLOCK to OUT.  The data
+ * for the record is taken from (DATA,DATELEN).  PK is the public key
+ * packet with the primary key. */
 static gpg_error_t
-print_pka_or_dane_records (iobuf_t out, kbnode_t keyblock, PKT_public_key *pk,
-                           const void *data, size_t datalen,
-                           int print_pka, int print_dane)
+print_dane_records (iobuf_t out, kbnode_t keyblock, PKT_public_key *pk,
+                    const void *data, size_t datalen)
 {
   gpg_error_t err = 0;
   kbnode_t kbctx, node;
@@ -1514,32 +1519,14 @@ print_pka_or_dane_records (iobuf_t out, kbnode_t keyblock, PKT_public_key *pk,
         continue;
 
       xfree (mbox);
-      mbox = mailbox_from_userid (uid->name);
+      mbox = mailbox_from_userid (uid->name, 0);
       if (!mbox)
         continue;
 
       domain = strchr (mbox, '@');
       *domain++ = 0;
 
-      if (print_pka)
-        {
-          es_fprintf (fp, "$ORIGIN _pka.%s.\n; %s\n; ", domain, hexfpr);
-          print_utf8_buffer (fp, uid->name, uid->len);
-          es_putc ('\n', fp);
-          gcry_md_hash_buffer (GCRY_MD_SHA1, hashbuf, mbox, strlen (mbox));
-          xfree (hash);
-          hash = zb32_encode (hashbuf, 8*20);
-          if (!hash)
-            {
-              err = gpg_error_from_syserror ();
-              goto leave;
-            }
-          len = strlen (hexfpr)/2;
-          es_fprintf (fp, "%s TYPE37 \\# %u 0006 0000 00 %02X %s\n\n",
-                      hash, 6 + len, len, hexfpr);
-        }
-
-      if (print_dane && hexdata)
+      if (1)
         {
           es_fprintf (fp, "$ORIGIN _openpgpkey.%s.\n; %s\n; ", domain, hexfpr);
           print_utf8_buffer (fp, uid->name, uid->len);
@@ -1628,7 +1615,7 @@ do_export_one_keyblock (ctrl_t ctrl, kbnode_t keyblock, u32 *keyid,
       if (node->pkt->pkttype == PKT_COMMENT)
         continue;
 
-      /* Skip ring trust packets - they should not ne here anyway.  */
+      /* Skip ring trust packets - they should not be here anyway.  */
       if (node->pkt->pkttype == PKT_RING_TRUST)
         continue;
 
@@ -1814,6 +1801,7 @@ do_export_one_keyblock (ctrl_t ctrl, kbnode_t keyblock, u32 *keyid,
                 {
                   if (gpg_err_code (err) == GPG_ERR_FULLY_CANCELED)
                     goto leave;
+                  write_status_error ("export_keys.secret", err);
                   skip_until_subkey = 1;
                   err = 0;
                 }
@@ -1914,17 +1902,16 @@ do_export_stream (ctrl_t ctrl, iobuf_t out, strlist_t users, int secret,
     stats = &dummystats;
   *any = 0;
   init_packet (&pkt);
-  kdbhd = keydb_new ();
+  kdbhd = keydb_new (ctrl);
   if (!kdbhd)
     return gpg_error_from_syserror ();
 
-  /* For the PKA and DANE format open a helper iobuf and for DANE
+  /* For the DANE format open a helper iobuf and
    * enforce some options.  */
-  if ((options & (EXPORT_PKA_FORMAT | EXPORT_DANE_FORMAT)))
+  if ((options & EXPORT_DANE_FORMAT))
     {
       out_help = iobuf_temp ();
-      if ((options & EXPORT_DANE_FORMAT))
-        options |= EXPORT_MINIMAL | EXPORT_CLEAN;
+      options |= EXPORT_MINIMAL | EXPORT_CLEAN;
     }
 
   if (!users)
@@ -2098,9 +2085,9 @@ do_export_stream (ctrl_t ctrl, iobuf_t out, strlist_t users, int secret,
           break;
         }
 
-      if (out_help)
+      if (out_help && (options & EXPORT_DANE_FORMAT))
         {
-          /* We want to write PKA or DANE records.  OUT_HELP has the
+          /* We want to write DANE records.  OUT_HELP has the
            * keyblock and we print a record for each uid to OUT. */
           const void *data;
           size_t datalen;
@@ -2109,10 +2096,7 @@ do_export_stream (ctrl_t ctrl, iobuf_t out, strlist_t users, int secret,
           data = iobuf_get_temp_buffer (out_help);
           datalen = iobuf_get_temp_length (out_help);
 
-          err = print_pka_or_dane_records (out,
-                                           keyblock, pk, data, datalen,
-                                           (options & EXPORT_PKA_FORMAT),
-                                           (options & EXPORT_DANE_FORMAT));
+          err = print_dane_records (out, keyblock, pk, data, datalen);
           if (err)
             goto leave;
 
@@ -2161,24 +2145,157 @@ key_to_sshblob (membuf_t *mb, const char *identifier, ...)
   va_start (arg_ptr, identifier);
   while ((a = va_arg (arg_ptr, gcry_mpi_t)))
     {
-      err = gcry_mpi_aprint (GCRYMPI_FMT_SSH, &buf, &buflen, a);
-      if (err)
-        break;
-      if (!strcmp (identifier, "ssh-ed25519")
-          && buflen > 5 && buf[4] == 0x40)
+      if (gcry_mpi_get_flag (a, GCRYMPI_FLAG_OPAQUE))
         {
-          /* We need to strip our 0x40 prefix.  */
-          put_membuf (mb, "\x00\x00\x00\x20", 4);
-          put_membuf (mb, buf+5, buflen-5);
+          unsigned int nbits;
+          const unsigned char *p;
+
+          p = gcry_mpi_get_opaque (a, &nbits);
+          buflen = (nbits + 7) / 8;
+
+          if (!strcmp (identifier, "ssh-ed25519")
+              && buflen > 1 && p[0] == 0x40)
+            {
+              /* We need to strip our 0x40 prefix.  */
+              put_membuf (mb, "\x00\x00\x00\x20", 4);
+              put_membuf (mb, p+1, buflen-1);
+            }
+          else
+            {
+              unsigned char c;
+
+              c = buflen >> 24;
+              put_membuf (mb, &c, 1);
+              c = buflen >> 16;
+              put_membuf (mb, &c, 1);
+              c = buflen >> 8;
+              put_membuf (mb, &c, 1);
+              c = buflen;
+              put_membuf (mb, &c, 1);
+              put_membuf (mb, p, buflen);
+            }
         }
       else
-        put_membuf (mb, buf, buflen);
-      gcry_free (buf);
+        {
+          err = gcry_mpi_aprint (GCRYMPI_FMT_SSH, &buf, &buflen, a);
+          if (err)
+            break;
+          put_membuf (mb, buf, buflen);
+          gcry_free (buf);
+        }
     }
   va_end (arg_ptr);
   return err;
 }
 
+
+static gpg_error_t
+export_one_ssh_key (estream_t fp, PKT_public_key *pk)
+{
+  gpg_error_t err;
+  const char *identifier = NULL;
+  membuf_t mb;
+  void *blob;
+  size_t bloblen;
+
+  init_membuf (&mb, 4096);
+
+  switch (pk->pubkey_algo)
+    {
+    case PUBKEY_ALGO_DSA:
+      identifier = "ssh-dss";
+      err = key_to_sshblob (&mb, identifier,
+                            pk->pkey[0], pk->pkey[1], pk->pkey[2], pk->pkey[3],
+                            NULL);
+      break;
+
+    case PUBKEY_ALGO_RSA:
+    case PUBKEY_ALGO_RSA_S:
+      identifier = "ssh-rsa";
+      err = key_to_sshblob (&mb, identifier, pk->pkey[1], pk->pkey[0], NULL);
+      break;
+
+    case PUBKEY_ALGO_ECDSA:
+      {
+        char *curveoid;
+        const char *curve;
+
+        curveoid = openpgp_oid_to_str (pk->pkey[0]);
+        if (!curveoid)
+          err = gpg_error_from_syserror ();
+        else if (!(curve = openpgp_oid_to_curve (curveoid, 0)))
+          err = gpg_error (GPG_ERR_UNKNOWN_CURVE);
+        else
+          {
+            if (!strcmp (curve, "nistp256"))
+              identifier = "ecdsa-sha2-nistp256";
+            else if (!strcmp (curve, "nistp384"))
+              identifier = "ecdsa-sha2-nistp384";
+            else if (!strcmp (curve, "nistp521"))
+              identifier = "ecdsa-sha2-nistp521";
+
+            if (!identifier)
+              err = gpg_error (GPG_ERR_UNKNOWN_CURVE);
+            else
+              err = key_to_sshblob (&mb, identifier, pk->pkey[1], NULL);
+          }
+        xfree (curveoid);
+      }
+      break;
+
+    case PUBKEY_ALGO_EDDSA:
+      if (openpgp_oid_is_ed25519 (pk->pkey[0]))
+        {
+          identifier = "ssh-ed25519";
+          err = key_to_sshblob (&mb, identifier, pk->pkey[1], NULL);
+        }
+      else if (openpgp_oid_is_ed448 (pk->pkey[0]))
+        {
+          identifier = "ssh-ed448";
+          err = key_to_sshblob (&mb, identifier, pk->pkey[1], NULL);
+        }
+      else
+        err = gpg_error (GPG_ERR_UNKNOWN_CURVE);
+      break;
+
+    case PUBKEY_ALGO_ELGAMAL_E:
+    case PUBKEY_ALGO_ELGAMAL:
+      err = gpg_error (GPG_ERR_UNUSABLE_PUBKEY);
+      break;
+
+    default:
+      err = GPG_ERR_PUBKEY_ALGO;
+      break;
+    }
+
+  if (err)
+    goto leave;
+
+  blob = get_membuf (&mb, &bloblen);
+  if (blob)
+    {
+      struct b64state b64_state;
+
+      es_fprintf (fp, "%s ", identifier);
+      err = b64enc_start_es (&b64_state, fp, "");
+      if (err)
+        {
+          xfree (blob);
+          goto leave;
+        }
+
+      err = b64enc_write (&b64_state, blob, bloblen);
+      b64enc_finish (&b64_state);
+
+      es_fprintf (fp, " openpgp:0x%08lX\n", (ulong)keyid_from_pk (pk, NULL));
+      xfree (blob);
+    }
+
+ leave:
+  xfree (get_membuf (&mb, NULL));
+  return err;
+}
+
 /* Export the key identified by USERID in the SSH public key format.
    The function exports the latest subkey with Authentication
    capability unless the '!' suffix is used to export a specific
@@ -2193,14 +2310,9 @@ export_ssh_key (ctrl_t ctrl, const char *userid)
   u32 curtime = make_timestamp ();
   kbnode_t latest_key, node;
   PKT_public_key *pk;
-  const char *identifier = NULL;
-  membuf_t mb;
   estream_t fp = NULL;
-  struct b64state b64_state;
   const char *fname = "-";
 
-  init_membuf (&mb, 4096);
-
   /* We need to know whether the key has been specified using the
      exact syntax ('!' suffix).  Thus we need to run a
      classify_user_id on our own.  */
@@ -2356,72 +2468,6 @@ export_ssh_key (ctrl_t ctrl, const char *userid)
   if (DBG_LOOKUP)
     log_debug ("\tusing key %08lX\n", (ulong) keyid_from_pk (pk, NULL));
 
-  switch (pk->pubkey_algo)
-    {
-    case PUBKEY_ALGO_DSA:
-      identifier = "ssh-dss";
-      err = key_to_sshblob (&mb, identifier,
-                            pk->pkey[0], pk->pkey[1], pk->pkey[2], pk->pkey[3],
-                            NULL);
-      break;
-
-    case PUBKEY_ALGO_RSA:
-    case PUBKEY_ALGO_RSA_S:
-      identifier = "ssh-rsa";
-      err = key_to_sshblob (&mb, identifier, pk->pkey[1], pk->pkey[0], NULL);
-      break;
-
-    case PUBKEY_ALGO_ECDSA:
-      {
-        char *curveoid;
-        const char *curve;
-
-        curveoid = openpgp_oid_to_str (pk->pkey[0]);
-        if (!curveoid)
-          err = gpg_error_from_syserror ();
-        else if (!(curve = openpgp_oid_to_curve (curveoid, 0)))
-          err = gpg_error (GPG_ERR_UNKNOWN_CURVE);
-        else
-          {
-            if (!strcmp (curve, "nistp256"))
-              identifier = "ecdsa-sha2-nistp256";
-            else if (!strcmp (curve, "nistp384"))
-              identifier = "ecdsa-sha2-nistp384";
-            else if (!strcmp (curve, "nistp521"))
-              identifier = "ecdsa-sha2-nistp521";
-
-            if (!identifier)
-              err = gpg_error (GPG_ERR_UNKNOWN_CURVE);
-            else
-              err = key_to_sshblob (&mb, identifier, pk->pkey[1], NULL);
-          }
-        xfree (curveoid);
-      }
-      break;
-
-    case PUBKEY_ALGO_EDDSA:
-      if (!openpgp_oid_is_ed25519 (pk->pkey[0]))
-        err = gpg_error (GPG_ERR_UNKNOWN_CURVE);
-      else
-        {
-          identifier = "ssh-ed25519";
-          err = key_to_sshblob (&mb, identifier, pk->pkey[1], NULL);
-        }
-      break;
-
-    case PUBKEY_ALGO_ELGAMAL_E:
-    case PUBKEY_ALGO_ELGAMAL:
-      err = gpg_error (GPG_ERR_UNUSABLE_PUBKEY);
-      break;
-
-    default:
-      err = GPG_ERR_PUBKEY_ALGO;
-      break;
-    }
-
-  if (!identifier)
-    goto leave;
-
   if (opt.outfile && *opt.outfile && strcmp (opt.outfile, "-"))
     fp = es_fopen ((fname = opt.outfile), "w");
   else
@@ -2433,32 +2479,15 @@ export_ssh_key (ctrl_t ctrl, const char *userid)
       goto leave;
     }
 
-  es_fprintf (fp, "%s ", identifier);
-  err = b64enc_start_es (&b64_state, fp, "");
-  if (!err)
-    {
-      void *blob;
-      size_t bloblen;
-
-      blob = get_membuf (&mb, &bloblen);
-      if (blob)
-        {
-          err = b64enc_write (&b64_state, blob, bloblen);
-          xfree (blob);
-          if (err)
-            goto leave;
-        }
-      err = b64enc_finish (&b64_state);
-    }
+  err = export_one_ssh_key (fp, pk);
   if (err)
     goto leave;
-  es_fprintf (fp, " openpgp:0x%08lX\n", (ulong)keyid_from_pk (pk, NULL));
 
   if (es_ferror (fp))
     err = gpg_error_from_syserror ();
   else
     {
-      if (es_fclose (fp))
+      if (fp != es_stdout && es_fclose (fp))
         err = gpg_error_from_syserror ();
       fp = NULL;
     }
@@ -2467,8 +2496,8 @@ export_ssh_key (ctrl_t ctrl, const char *userid)
     log_error (_("error writing '%s': %s\n"), fname, gpg_strerror (err));
 
  leave:
-  es_fclose (fp);
-  xfree (get_membuf (&mb, NULL));
+  if (fp != es_stdout)
+    es_fclose (fp);
   release_kbnode (keyblock);
   return err;
 }
