@@ -15,7 +15,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, see <http://www.gnu.org/licenses/>.
+ * along with this program; if not, see <https://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -29,14 +29,14 @@
 #endif
 
 #include "gpg.h"
-#include "util.h"
+#include "../common/util.h"
 #include "options.h"
 #include "packet.h"
-#include "ttyio.h"
+#include "../common/ttyio.h"
 #include "filter.h"
 #include "main.h"
-#include "status.h"
-#include "i18n.h"
+#include "../common/status.h"
+#include "../common/i18n.h"
 
 
 /* Get the output filename.  On success, the actual filename that is
@@ -70,7 +70,8 @@ get_output_file (const byte *embedded_name, int embedded_namelen,
           goto leave;
         }
     }
-  else if (opt.outfile)
+  else if (opt.outfile
+           && !(opt.flags.use_embedded_filename && opt.flags.dummy_outfile))
     {
       fname = xtrystrdup (opt.outfile);
       if (!fname)
@@ -108,9 +109,25 @@ get_output_file (const byte *embedded_name, int embedded_namelen,
     }
   else if (iobuf_is_pipe_filename (fname) || !*fname)
     {
-      /* No filename or "-" given; write to stdout. */
-      fp = es_stdout;
-      es_set_binary (fp);
+      /* Special file name, no filename, or "-" given; write to the
+       * file descriptor or to stdout. */
+      int fd;
+      char xname[64];
+
+      fd = check_special_filename (fname, 1, 0);
+      if (fd == -1)
+        {
+          /* Not a special filename, thus we want stdout.  */
+          fp = es_stdout;
+          es_set_binary (fp);
+        }
+      else if (!(fp = es_fdopen_nc (fd, "wb")))
+        {
+          err = gpg_error_from_syserror ();
+          snprintf (xname, sizeof xname, "[fd %d]", fd);
+          log_error (_("can't open '%s': %s\n"), xname, gpg_strerror (err));
+          goto leave;
+        }
     }
   else
     {
@@ -129,7 +146,6 @@ get_output_file (const byte *embedded_name, int embedded_namelen,
 	}
     }
 
-#ifndef __riscos__
   if (opt.outfp && is_secured_file (es_fileno (opt.outfp)))
     {
       err = gpg_error (GPG_ERR_EPERM);
@@ -151,44 +167,6 @@ get_output_file (const byte *embedded_name, int embedded_namelen,
       log_error (_("error creating '%s': %s\n"), fname, gpg_strerror (err));
       goto leave;
     }
-#else /* __riscos__ */
-  /* If no output filename was given, i.e. we constructed it, convert
-     all '.' in fname to '/' but not vice versa as we don't create
-     directories! */
-  if (!opt.outfile)
-    for (c = 0; fname[c]; ++c)
-      if (fname[c] == '.')
-	fname[c] = '/';
-
-  if (fp || nooutput)
-    ;
-  else
-    {
-      /* Note: riscos stuff is not expected to work anymore.  If we
-         want to port it again to riscos we should do most of the suff
-         in estream.  FIXME: Consider to remove all riscos special
-         cases.  */
-      fp = fopen (fname, "wb");
-      if (!fp)
-	{
-	  log_error (_("error creating '%s': %s\n"), fname, gpg_strerror (err));
-	  err = GPG_ERR_CREATE_FILE;
-	  if (errno == 106)
-	    log_info ("Do output file and input file have the same name?\n");
-	  goto leave;
-	}
-
-      /* If there's a ,xxx extension in the embedded filename,
-         use that, else check whether the user input (in fname)
-         has a ,xxx appended, then use that in preference */
-      if ((c = riscos_get_filetype_from_string (embedded_name,
-                                                embedded_namelen)) != -1)
-	filetype = c;
-      if ((c = riscos_get_filetype_from_string (fname, strlen (fname))) != -1)
-	filetype = c;
-      riscos_set_filetype_by_number (fname, filetype);
-    }
-#endif /* __riscos__ */
 
  leave:
   if (err)
