@@ -39,9 +39,49 @@
 #include "tofu.h"
 #include "key-clean.h"
 
-static void write_record (ctrl_t ctrl, TRUSTREC *rec);
-static void do_sync(void);
+static u32
+keyid_from_fpr20 (ctrl_t ctrl, const byte *fpr, u32 *keyid)
+{
+  u32 dummy_keyid[2];
+  int fprlen;
 
+  if( !keyid )
+    keyid = dummy_keyid;
+
+  /* Problem: We do only use fingerprints in the trustdb but
+   * we need the keyID here to indetify the key; we can only
+   * use that ugly hack to distinguish between 16 and 20
+   * bytes fpr - it does not work always so we better change
+   * the whole validation code to only work with
+   * fingerprints */
+  fprlen = (!fpr[16] && !fpr[17] && !fpr[18] && !fpr[19])? 16:20;
+
+  if (fprlen != 20)
+    {
+      /* This is special as we have to lookup the key first.  */
+      PKT_public_key pk;
+      int rc;
+
+      memset (&pk, 0, sizeof pk);
+      rc = get_pubkey_byfprint (ctrl, &pk, NULL, fpr, fprlen);
+      if (rc)
+        {
+          log_printhex (fpr, fprlen,
+                        "Oops: keyid_from_fingerprint: no pubkey; fpr:");
+          keyid[0] = 0;
+          keyid[1] = 0;
+        }
+      else
+        keyid_from_pk (&pk, keyid);
+    }
+  else
+    {
+      keyid[0] = buf32_to_u32 (fpr+12);
+      keyid[1] = buf32_to_u32 (fpr+16);
+    }
+
+  return keyid[1];
+}
 
 typedef struct key_item **KeyHashTable; /* see new_key_hash_table() */
 
@@ -192,7 +232,7 @@ release_key_array ( struct key_array *keys )
  * before initializing the validation module.
  * FIXME: Should be replaced by a function to add those keys to the trustdb.
  */
-static void
+void
 tdb_register_trusted_keyid (u32 *keyid)
 {
   struct key_item *k;
@@ -217,15 +257,21 @@ tdb_register_trusted_key (const char *string)
     {
       if (desc.mode == KEYDB_SEARCH_MODE_LONG_KID)
         {
-          tdb_register_trusted_keyid (desc.u.kid);
+          register_trusted_keyid (desc.u.kid);
           return;
         }
-      if (desc.mode == KEYDB_SEARCH_MODE_FPR
-          || desc.mode == KEYDB_SEARCH_MODE_FPR20)
+      if (desc.mode == KEYDB_SEARCH_MODE_FPR && desc.fprlen == 20)
         {
           kid[0] = buf32_to_u32 (desc.u.fpr+12);
           kid[1] = buf32_to_u32 (desc.u.fpr+16);
-          tdb_register_trusted_keyid (kid);
+          register_trusted_keyid (kid);
+          return;
+        }
+      if (desc.mode == KEYDB_SEARCH_MODE_FPR && desc.fprlen == 32)
+        {
+          kid[0] = buf32_to_u32 (desc.u.fpr);
+          kid[1] = buf32_to_u32 (desc.u.fpr+4);
+          register_trusted_keyid (kid);
           return;
         }
     }
@@ -257,49 +303,6 @@ add_utk (u32 *kid)
 }
 
 
-/* Add/remove KID to/from the list of ultimately trusted keys.  */
-void
-tdb_update_utk (u32 *kid, int add)
-{
-  struct key_item *k, *k_prev;
-
-  k_prev = NULL;
-  for (k = utk_list; k; k = k->next)
-    if (k->kid[0] == kid[0] && k->kid[1] == kid[1])
-      break;
-    else
-      k_prev = k;
-
-  if (add)
-    {
-      if (!k)
-        {
-          k = new_key_item ();
-          k->kid[0] = kid[0];
-          k->kid[1] = kid[1];
-          k->ownertrust = TRUST_ULTIMATE;
-          k->next = utk_list;
-          utk_list = k;
-          if ( opt.verbose > 1 )
-            log_info(_("key %s: accepted as trusted key\n"), keystr(kid));
-        }
-    }
-  else
-    {
-      if (k)
-        {
-          if (k_prev)
-            k_prev->next = k->next;
-          else
-            utk_list = NULL;
-
-          xfree (k->trust_regexp);
-          xfree (k);
-        }
-    }
-}
-
-
 /****************
  * Verify that all our secret keys are usable and put them into the utk_list.
  */
@@ -309,59 +312,26 @@ verify_own_keys (ctrl_t ctrl)
   TRUSTREC rec;
   ulong recnum;
   int rc;
-  struct key_item *k, *k2;
-  int need_revalidation = 0;
+  struct key_item *k;
 
   if (utk_list)
-    return; /* Has already been run.  */
+    return;
 
   /* scan the trustdb to find all ultimately trusted keys */
   for (recnum=1; !tdbio_read_record (recnum, &rec, 0); recnum++ )
     {
-      if ( rec.rectype == RECTYPE_TRUST
-           && (rec.r.trust.ownertrust & TRUST_MASK) == TRUST_ULTIMATE)
+      if (rec.rectype == RECTYPE_TRUST
+          && (rec.r.trust.ownertrust & TRUST_MASK) == TRUST_ULTIMATE)
         {
-          byte *fpr = rec.r.trust.fingerprint;
-          int fprlen;
           u32 kid[2];
 
-          /* Problem: We do only use fingerprints in the trustdb but
-           * we need the keyID here to indetify the key; we can only
-           * use that ugly hack to distinguish between 16 and 20 bytes
-           * fpr - it does not work always so we better change the
-           * whole validation code to only work with fingerprints */
-          fprlen = (!fpr[16] && !fpr[17] && !fpr[18] && !fpr[19])? 16:20;
-          keyid_from_fingerprint (ctrl, fpr, fprlen, kid);
+          keyid_from_fpr20 (ctrl, rec.r.trust.fingerprint, kid);
           if (!add_utk (kid))
-            log_info(_("key %s occurs more than once in the trustdb\n"),
-                     keystr(kid));
-          else if ((rec.r.trust.flags & 1))
-            {
-              /* Record marked as inserted via --trusted-key.  Is this
-               * still the case?  */
-              for (k2 = user_utk_list; k2; k2 = k2->next)
-                if (k2->kid[0] == kid[0] && k2->kid[1] == kid[1])
-                  break;
-              if (!k2) /* No - clear the flag.  */
-                {
-                  if (DBG_TRUST)
-                    log_debug ("clearing former --trusted-key %s\n",
-                               keystr (kid));
-                  rec.r.trust.ownertrust = TRUST_UNKNOWN;
-                  rec.r.trust.flags &= ~(rec.r.trust.flags & 1);
-                  write_record (ctrl, &rec);
-                  need_revalidation = 1;
-                }
-            }
+            log_info (_("key %s occurs more than once in the trustdb\n"),
+                      keystr(kid));
         }
     }
 
-  if (need_revalidation)
-    {
-      tdb_revalidation_mark (ctrl);
-      do_sync ();
-    }
-
   /* Put any --trusted-key keys into the trustdb */
   for (k = user_utk_list; k; k = k->next)
     {
@@ -370,7 +340,7 @@ verify_own_keys (ctrl_t ctrl)
           PKT_public_key pk;
 
           memset (&pk, 0, sizeof pk);
-          rc = get_pubkey_with_ldap_fallback (ctrl, &pk, k->kid);
+          rc = get_pubkey (ctrl, &pk, k->kid);
           if (rc)
 	    log_info(_("key %s: no public key for trusted key - skipped\n"),
 		     keystr(k->kid));
@@ -378,7 +348,7 @@ verify_own_keys (ctrl_t ctrl)
 	    {
 	      tdb_update_ownertrust
                 (ctrl, &pk, ((tdb_get_ownertrust (ctrl, &pk, 0) & ~TRUST_MASK)
-                             | TRUST_ULTIMATE ),  1);
+                             | TRUST_ULTIMATE ));
 	      release_public_key_parts (&pk);
 	    }
 
@@ -687,7 +657,7 @@ tdb_check_or_update (ctrl_t ctrl)
       if (opt.interactive)
 	update_trustdb (ctrl);
       else if (!opt.no_auto_check_trustdb)
-        check_trustdb (ctrl);
+	check_trustdb (ctrl);
     }
 }
 
@@ -819,8 +789,7 @@ tdb_get_min_ownertrust (ctrl_t ctrl, PKT_public_key *pk, int no_create)
  * The key should be a primary one.
  */
 void
-tdb_update_ownertrust (ctrl_t ctrl, PKT_public_key *pk, unsigned int new_trust,
-                       int as_trusted_key)
+tdb_update_ownertrust (ctrl_t ctrl, PKT_public_key *pk, unsigned int new_trust )
 {
   TRUSTREC rec;
   gpg_error_t err;
@@ -832,24 +801,11 @@ tdb_update_ownertrust (ctrl_t ctrl, PKT_public_key *pk, unsigned int new_trust,
   if (!err)
     {
       if (DBG_TRUST)
-        log_debug ("update ownertrust from %u to %u%s\n",
-                   (unsigned int)rec.r.trust.ownertrust, new_trust,
-                   as_trusted_key? " via --trusted-key":"");
+        log_debug ("update ownertrust from %u to %u\n",
+                   (unsigned int)rec.r.trust.ownertrust, new_trust );
       if (rec.r.trust.ownertrust != new_trust)
         {
           rec.r.trust.ownertrust = new_trust;
-          /* Clear or set the trusted key flag if the new value is
-           * ultimate.  This is required so that we know which keys
-           * have been added by --trusted-keys.  */
-          if ((rec.r.trust.ownertrust & TRUST_MASK) == TRUST_ULTIMATE)
-            {
-              if (as_trusted_key)
-                rec.r.trust.flags |= 1;
-              else
-                rec.r.trust.flags &= ~(rec.r.trust.flags & 1);
-            }
-          else
-            rec.r.trust.flags &= ~(rec.r.trust.flags & 1);
           write_record (ctrl, &rec);
           tdb_revalidation_mark (ctrl);
           do_sync ();
@@ -857,20 +813,14 @@ tdb_update_ownertrust (ctrl_t ctrl, PKT_public_key *pk, unsigned int new_trust,
     }
   else if (gpg_err_code (err) == GPG_ERR_NOT_FOUND)
     { /* no record yet - create a new one */
-      size_t dummy;
-
       if (DBG_TRUST)
-        log_debug ("insert ownertrust %u%s\n", new_trust,
-                   as_trusted_key? " via --trusted-key":"");
+        log_debug ("insert ownertrust %u\n", new_trust );
 
       memset (&rec, 0, sizeof rec);
       rec.recnum = tdbio_new_recnum (ctrl);
       rec.rectype = RECTYPE_TRUST;
-      fingerprint_from_pk (pk, rec.r.trust.fingerprint, &dummy);
+      fpr20_from_pk (pk, rec.r.trust.fingerprint);
       rec.r.trust.ownertrust = new_trust;
-      if ((rec.r.trust.ownertrust & TRUST_MASK) == TRUST_ULTIMATE
-          && as_trusted_key)
-        rec.r.trust.flags = 1;
       write_record (ctrl, &rec);
       tdb_revalidation_mark (ctrl);
       do_sync ();
@@ -919,15 +869,13 @@ update_min_ownertrust (ctrl_t ctrl, u32 *kid, unsigned int new_trust)
     }
   else if (gpg_err_code (err) == GPG_ERR_NOT_FOUND)
     { /* no record yet - create a new one */
-      size_t dummy;
-
       if (DBG_TRUST)
         log_debug ("insert min_ownertrust %u\n", new_trust );
 
       memset (&rec, 0, sizeof rec);
       rec.recnum = tdbio_new_recnum (ctrl);
       rec.rectype = RECTYPE_TRUST;
-      fingerprint_from_pk (pk, rec.r.trust.fingerprint, &dummy);
+      fpr20_from_pk (pk, rec.r.trust.fingerprint);
       rec.r.trust.min_ownertrust = new_trust;
       write_record (ctrl, &rec);
       tdb_revalidation_mark (ctrl);
@@ -944,7 +892,6 @@ update_min_ownertrust (ctrl_t ctrl, u32 *kid, unsigned int new_trust)
 
 /*
  * Clear the ownertrust and min_ownertrust values.
- * Also schedule a revalidation if a stale validity record exists.
  *
  * Return: True if a change actually happened.
  */
@@ -978,26 +925,6 @@ tdb_clear_ownertrusts (ctrl_t ctrl, PKT_public_key *pk)
           do_sync ();
           return 1;
         }
-      else
-        {
-          /* Check whether we have a stale RECTYPE_VALID for that key
-           * and if its validity ist set, schedule a revalidation.  */
-          ulong recno = rec.r.trust.validlist;
-          while (recno)
-            {
-              read_record (recno, &rec, RECTYPE_VALID);
-              if (rec.r.valid.validity)
-                break;
-              recno = rec.r.valid.next;
-            }
-          if (recno)
-            {
-              if (DBG_TRUST)
-                log_debug ("stale validity value detected"
-                           " - scheduling check\n");
-              tdb_revalidation_mark (ctrl);
-            }
-        }
     }
   else if (gpg_err_code (err) != GPG_ERR_NOT_FOUND)
     {
@@ -1028,12 +955,10 @@ update_validity (ctrl_t ctrl, PKT_public_key *pk, PKT_user_id *uid,
   if (gpg_err_code (err) == GPG_ERR_NOT_FOUND)
     {
       /* No record yet - create a new one. */
-      size_t dummy;
-
       memset (&trec, 0, sizeof trec);
       trec.recnum = tdbio_new_recnum (ctrl);
       trec.rectype = RECTYPE_TRUST;
-      fingerprint_from_pk (pk, trec.r.trust.fingerprint, &dummy);
+      fpr20_from_pk (pk, trec.r.trust.fingerprint);
       trec.r.trust.ownertrust = 0;
       }
 
@@ -1251,7 +1176,7 @@ tdb_get_validity_core (ctrl_t ctrl,
           if (sig && sig->signers_uid)
             /* Make sure the UID matches.  */
             {
-              char *email = mailbox_from_userid (user_id->name);
+              char *email = mailbox_from_userid (user_id->name, 0);
               if (!email || !*email || strcmp (sig->signers_uid, email) != 0)
                 {
                   if (DBG_TRUST)
@@ -1512,7 +1437,7 @@ ask_ownertrust (ctrl_t ctrl, u32 *kid, int minimum)
     {
       log_info("force trust for key %s to %s\n",
 	       keystr(kid),trust_value_to_string(opt.force_ownertrust));
-      tdb_update_ownertrust (ctrl, pk, opt.force_ownertrust, 0);
+      tdb_update_ownertrust (ctrl, pk, opt.force_ownertrust);
       ot=opt.force_ownertrust;
     }
   else
@@ -1690,50 +1615,38 @@ sanitize_regexp(const char *old)
   return new;
 }
 
-
 /* Used by validate_one_keyblock to confirm a regexp within a trust
- * signature.  Returns 1 for match, and 0 for no match or regex
- * error. */
+   signature.  Returns 1 for match, and 0 for no match or regex
+   error. */
 static int
-check_regexp (const char *expr,const char *string)
+check_regexp(const char *expr,const char *string)
 {
   int ret;
   char *regexp;
-  char *stringbuf = NULL;
-  regex_t pat;
 
-  regexp = sanitize_regexp (expr);
+  regexp=sanitize_regexp(expr);
 
-  ret = regcomp (&pat, regexp, (REG_ICASE|REG_EXTENDED));
-  if (!ret)
-    {
-      if (*regexp == '<' && !strchr (string, '<')
-          && is_valid_mailbox (string))
-        {
-          /* The R.E. starts with an angle bracket but STRING seems to
-           * be a plain mailbox (e.g. "foo@example.org").  The
-           * commonly used R.E. pattern "<[^>]+[@.]example\.org>$"
-           * won't be able to detect this.  Thus we enclose STRING
-           * into angle brackets for checking.  */
-          stringbuf = xstrconcat ("<", string, ">", NULL);
-          string = stringbuf;
-        }
-      ret = regexec (&pat, string, 0, NULL, 0);
-      regfree (&pat);
-    }
+  {
+    regex_t pat;
 
-  ret = !ret;
+    ret=regcomp(&pat,regexp,REG_ICASE|REG_EXTENDED);
+    if(ret==0)
+      {
+	ret=regexec(&pat,string,0,NULL,0);
+	regfree(&pat);
+      }
+    ret=(ret==0);
+  }
 
-  if (DBG_TRUST)
-    log_debug ("regexp '%s' ('%s') on '%s'%s: %s\n",
-               regexp, expr, string, stringbuf? " (fixed)":"", ret? "YES":"NO");
+  if(DBG_TRUST)
+    log_debug("regexp '%s' ('%s') on '%s': %s\n",
+	      regexp,expr,string,ret?"YES":"NO");
+
+  xfree(regexp);
 
-  xfree (regexp);
-  xfree (stringbuf);
   return ret;
 }
 
-
 /*
  * Return true if the key is signed by one of the keys in the given
  * key ID list.  User IDs with a valid signature are marked by node
@@ -1921,7 +1834,7 @@ search_skipfnc (void *opaque, u32 *kid, int dummy_uid_no)
 
 /*
  * Scan all keys and return a key_array of all suitable keys from
- * kllist.  The caller has to pass keydb handle so that we don't use
+ * klist.  The caller has to pass keydb handle so that we don't use
  * to create our own.  Returns either a key_array or NULL in case of
  * an error.  No results found are indicated by an empty array.
  * Caller hast to release the returned array.
@@ -2133,7 +2046,7 @@ validate_keys (ctrl_t ctrl, int interactive)
      trust. */
   keydb_rebuild_caches (ctrl, 0);
 
-  kdb = keydb_new ();
+  kdb = keydb_new (ctrl);
   if (!kdb)
     return gpg_error_from_syserror ();
 
