@@ -28,7 +28,6 @@
 #include <errno.h>
 #include <unistd.h>
 #include <time.h>
-#include <assert.h>
 
 #include "gpgsm.h"
 #include <gcrypt.h>
@@ -147,7 +146,7 @@ init_dek (DEK dek)
   return 0;
 }
 
-/* Encrypt an RSA session key.  */
+/* Encode an RSA session key.  */
 static int
 encode_session_key (DEK dek, gcry_sexp_t * r_data)
 {
@@ -185,7 +184,7 @@ ecdh_encrypt (DEK dek, gcry_sexp_t s_pkey, gcry_sexp_t *r_encval)
   const char *encr_algo_str;
   const char *wrap_algo_str;
   int hash_algo, cipher_algo;
-  unsigned int keylen, hashlen;
+  unsigned int keylen;
   unsigned char key[32];
   gcry_sexp_t s_data = NULL;
   gcry_sexp_t s_encr = NULL;
@@ -230,20 +229,28 @@ ecdh_encrypt (DEK dek, gcry_sexp_t s_pkey, gcry_sexp_t *r_encval)
   curvebuf = NULL;
 
   /* Our mapping matches the recommended algorithms from RFC-5753 but
-   * not supporing the short curves which would require 3DES.  */
+   * not supporting the short curves which would require 3DES.  */
   if (curvebits < 255)
     {
       err = gpg_error (GPG_ERR_UNKNOWN_CURVE);
       log_error ("%s: curve '%s' is not supported\n", __func__, curve);
       goto leave;
     }
+  else if (opt.force_ecdh_sha1kdf)
+    {
+      /* dhSinglePass-stdDH-sha1kdf-scheme */
+      encr_algo_str = "1.3.133.16.840.63.0.2";
+      wrap_algo_str = "2.16.840.1.101.3.4.1.45";
+      hash_algo     = GCRY_MD_SHA1;
+      cipher_algo   = GCRY_CIPHER_AES256;
+      keylen        = 32;
+    }
   else if (curvebits <= 256)
     {
       /* dhSinglePass-stdDH-sha256kdf-scheme */
       encr_algo_str = "1.3.132.1.11.1";
       wrap_algo_str = "2.16.840.1.101.3.4.1.5";
       hash_algo     = GCRY_MD_SHA256;
-      hashlen       = 32;
       cipher_algo   = GCRY_CIPHER_AES128;
       keylen        = 16;
     }
@@ -253,7 +260,6 @@ ecdh_encrypt (DEK dek, gcry_sexp_t s_pkey, gcry_sexp_t *r_encval)
       encr_algo_str = "1.3.132.1.11.2";
       wrap_algo_str = "2.16.840.1.101.3.4.1.25";
       hash_algo     = GCRY_MD_SHA384;
-      hashlen       = 48;
       cipher_algo   = GCRY_CIPHER_AES256;
       keylen        = 24;
     }
@@ -263,7 +269,6 @@ ecdh_encrypt (DEK dek, gcry_sexp_t s_pkey, gcry_sexp_t *r_encval)
       encr_algo_str = "1.3.132.1.11.3";
       wrap_algo_str = "2.16.840.1.101.3.4.1.45";
       hash_algo     = GCRY_MD_SHA512;
-      hashlen       = 64;
       cipher_algo   = GCRY_CIPHER_AES256;
       keylen        = 32;
     }
@@ -338,32 +343,10 @@ ecdh_encrypt (DEK dek, gcry_sexp_t s_pkey, gcry_sexp_t *r_encval)
   if (DBG_CRYPTO)
     log_printhex (secret, secretlen, "ECDH X ..:");
 
-  /* Derive a KEK (key wrapping key) using MESSAGE and SECRET_X.
-   * According to SEC1 3.6.1 we should check that
-   *   SECRETLEN + UKMLEN + 4 < maxhashlen
-   * However, we have no practical limit on the hash length and thus
-   * there is no point in checking this.  The second check that
-   *   KEYLEN < hashlen*(2^32-1)
-   * is obviously also not needed.  Because with our allowed
-   * parameters KEYLEN is always less or equal to HASHLEN so that we
-   * do not need to iterate at all.
-   */
-  log_assert (gcry_md_get_algo_dlen (hash_algo) == hashlen);
-  {
-    gcry_md_hd_t hash_hd;
-    err = gcry_md_open (&hash_hd, hash_algo, 0);
-    if (err)
-      goto leave;
-    gcry_md_write(hash_hd, secret, secretlen);
-    gcry_md_write(hash_hd, "\x00\x00\x00\x01", 4);  /* counter */
-    err = hash_ecc_cms_shared_info (hash_hd, wrap_algo_str, keylen, NULL, 0);
-    gcry_md_final (hash_hd);
-    log_assert (keylen <= sizeof key && keylen <= hashlen);
-    memcpy (key, gcry_md_read (hash_hd, 0), keylen);
-    gcry_md_close (hash_hd);
-    if (err)
-      goto leave;
-  }
+  err = ecdh_derive_kek (key, keylen, hash_algo, wrap_algo_str,
+                         secret, secretlen, NULL, 0);
+  if (err)
+    goto leave;
 
   if (DBG_CRYPTO)
     log_printhex (key, keylen, "KEK .....:");
@@ -483,10 +466,7 @@ encrypt_dek (const DEK dek, ksba_cert_t cert, int pk_algo,
   s_data = NULL; /* (avoid compiler warning) */
   if (pk_algo == GCRY_PK_ECC)
     {
-      if (!(opt.compat_flags & COMPAT_ALLOW_ECC_ENCR))
-        rc = gpg_error (GPG_ERR_NOT_SUPPORTED);
-      else
-        rc = ecdh_encrypt (dek, s_pkey, &s_ciph);
+      rc = ecdh_encrypt (dek, s_pkey, &s_ciph);
     }
   else
     {
@@ -595,8 +575,9 @@ encrypt_cb (void *cb_value, char *buffer, size_t count, size_t *nread)
 int
 gpgsm_encrypt (ctrl_t ctrl, certlist_t recplist, int data_fd, estream_t out_fp)
 {
-  gpg_error_t err = 0;
+  int rc = 0;
   gnupg_ksba_io_t b64writer = NULL;
+  gpg_error_t err;
   ksba_writer_t writer;
   ksba_reader_t reader = NULL;
   ksba_cms_t cms = NULL;
@@ -625,7 +606,7 @@ gpgsm_encrypt (ctrl_t ctrl, certlist_t recplist, int data_fd, estream_t out_fp)
       log_error(_("no valid recipients given\n"));
       gpgsm_status (ctrl, STATUS_NO_RECP, "0");
       audit_log_i (ctrl->audit, AUDIT_GOT_RECIPIENTS, 0);
-      err = gpg_error (GPG_ERR_NO_PUBKEY);
+      rc = gpg_error (GPG_ERR_NO_PUBKEY);
       goto leave;
     }
 
@@ -633,11 +614,11 @@ gpgsm_encrypt (ctrl_t ctrl, certlist_t recplist, int data_fd, estream_t out_fp)
     count++;
   audit_log_i (ctrl->audit, AUDIT_GOT_RECIPIENTS, count);
 
-  kh = keydb_new ();
+  kh = keydb_new (ctrl);
   if (!kh)
     {
       log_error (_("failed to allocate keyDB handle\n"));
-      err = gpg_error (GPG_ERR_GENERAL);
+      rc = gpg_error (GPG_ERR_GENERAL);
       goto leave;
     }
 
@@ -645,43 +626,45 @@ gpgsm_encrypt (ctrl_t ctrl, certlist_t recplist, int data_fd, estream_t out_fp)
   data_fp = es_fdopen_nc (data_fd, "rb");
   if (!data_fp)
     {
-      err = gpg_error_from_syserror ();
-      log_error ("fdopen() failed: %s\n", gpg_strerror (err));
+      rc = gpg_error_from_syserror ();
+      log_error ("fdopen() failed: %s\n", strerror (errno));
       goto leave;
     }
 
   err = ksba_reader_new (&reader);
-  if (!err)
-    err = ksba_reader_set_cb (reader, encrypt_cb, &encparm);
   if (err)
-    goto leave;
+      rc = err;
+  if (!rc)
+    rc = ksba_reader_set_cb (reader, encrypt_cb, &encparm);
+  if (rc)
+      goto leave;
 
   encparm.fp = data_fp;
 
   ctrl->pem_name = "ENCRYPTED MESSAGE";
-  err = gnupg_ksba_create_writer
+  rc = gnupg_ksba_create_writer
     (&b64writer, ((ctrl->create_pem? GNUPG_KSBA_IO_PEM : 0)
                   | (ctrl->create_base64? GNUPG_KSBA_IO_BASE64 : 0)),
      ctrl->pem_name, out_fp, &writer);
-  if (err)
+  if (rc)
     {
-      log_error ("can't create writer: %s\n", gpg_strerror (err));
+      log_error ("can't create writer: %s\n", gpg_strerror (rc));
       goto leave;
     }
 
-  gnupg_ksba_set_progress_cb (b64writer, gpgsm_progress_cb, ctrl);
-  if (ctrl->input_size_hint)
-    gnupg_ksba_set_total (b64writer, ctrl->input_size_hint);
-
   err = ksba_cms_new (&cms);
   if (err)
-    goto leave;
+    {
+      rc = err;
+      goto leave;
+    }
 
   err = ksba_cms_set_reader_writer (cms, reader, writer);
   if (err)
     {
       log_error ("ksba_cms_set_reader_writer failed: %s\n",
                  gpg_strerror (err));
+      rc = err;
       goto leave;
     }
 
@@ -696,6 +679,7 @@ gpgsm_encrypt (ctrl_t ctrl, certlist_t recplist, int data_fd, estream_t out_fp)
     {
       log_error ("ksba_cms_set_content_type failed: %s\n",
                  gpg_strerror (err));
+      rc = err;
       goto leave;
     }
 
@@ -707,34 +691,34 @@ gpgsm_encrypt (ctrl_t ctrl, certlist_t recplist, int data_fd, estream_t out_fp)
       log_error (_("cipher algorithm '%s' may not be used in %s mode\n"),
 		 opt.def_cipher_algoid,
 		 gnupg_compliance_option_string (opt.compliance));
-      err = gpg_error (GPG_ERR_CIPHER_ALGO);
+      rc = gpg_error (GPG_ERR_CIPHER_ALGO);
       goto leave;
     }
 
   if (!gnupg_rng_is_compliant (opt.compliance))
     {
-      err = gpg_error (GPG_ERR_FORBIDDEN);
+      rc = gpg_error (GPG_ERR_FORBIDDEN);
       log_error (_("%s is not compliant with %s mode\n"),
                  "RNG",
                  gnupg_compliance_option_string (opt.compliance));
       gpgsm_status_with_error (ctrl, STATUS_ERROR,
-                               "random-compliance", err);
+                               "random-compliance", rc);
       goto leave;
     }
 
   /* Create a session key */
   dek = xtrycalloc_secure (1, sizeof *dek);
   if (!dek)
-    err = gpg_error_from_syserror ();
+    rc = out_of_core ();
   else
-    {
-      dek->algoid = opt.def_cipher_algoid;
-      err = init_dek (dek);
-    }
-  if (err)
+  {
+    dek->algoid = opt.def_cipher_algoid;
+    rc = init_dek (dek);
+  }
+  if (rc)
     {
       log_error ("failed to create the session key: %s\n",
-                 gpg_strerror (err));
+                 gpg_strerror (rc));
       goto leave;
     }
 
@@ -743,6 +727,7 @@ gpgsm_encrypt (ctrl_t ctrl, certlist_t recplist, int data_fd, estream_t out_fp)
     {
       log_error ("ksba_cms_set_content_enc_algo failed: %s\n",
                  gpg_strerror (err));
+      rc = err;
       goto leave;
     }
 
@@ -752,7 +737,7 @@ gpgsm_encrypt (ctrl_t ctrl, certlist_t recplist, int data_fd, estream_t out_fp)
   encparm.buffer = xtrymalloc (encparm.bufsize);
   if (!encparm.buffer)
     {
-      err = gpg_error_from_syserror ();
+      rc = out_of_core ();
       goto leave;
     }
 
@@ -768,12 +753,11 @@ gpgsm_encrypt (ctrl_t ctrl, certlist_t recplist, int data_fd, estream_t out_fp)
       unsigned char *encval;
       unsigned int nbits;
       int pk_algo;
-      char *curve = NULL;
 
       /* Check compliance.  */
-      pk_algo = gpgsm_get_key_algo_info (cl->cert, &nbits, &curve);
+      pk_algo = gpgsm_get_key_algo_info (cl->cert, &nbits);
       if (!gnupg_pk_is_compliant (opt.compliance, pk_algo, 0,
-                                  NULL, nbits, curve))
+                                  NULL, nbits, NULL))
         {
           char  kidstr[10+1];
 
@@ -788,18 +772,15 @@ gpgsm_encrypt (ctrl_t ctrl, certlist_t recplist, int data_fd, estream_t out_fp)
       /* Fixme: When adding ECC we need to provide the curvename and
        * the key to gnupg_pk_is_compliant.  */
       if (compliant
-          && !gnupg_pk_is_compliant (CO_DE_VS, pk_algo, 0, NULL, nbits, curve))
+          && !gnupg_pk_is_compliant (CO_DE_VS, pk_algo, 0, NULL, nbits, NULL))
         compliant = 0;
 
-      xfree (curve);
-      curve = NULL;
-
-      err = encrypt_dek (dek, cl->cert, pk_algo, &encval);
-      if (err)
+      rc = encrypt_dek (dek, cl->cert, pk_algo, &encval);
+      if (rc)
         {
-          audit_log_cert (ctrl->audit, AUDIT_ENCRYPTED_TO, cl->cert, err);
+          audit_log_cert (ctrl->audit, AUDIT_ENCRYPTED_TO, cl->cert, rc);
           log_error ("encryption failed for recipient no. %d: %s\n",
-                     recpno, gpg_strerror (err));
+                     recpno, gpg_strerror (rc));
           goto leave;
         }
 
@@ -809,6 +790,7 @@ gpgsm_encrypt (ctrl_t ctrl, certlist_t recplist, int data_fd, estream_t out_fp)
           audit_log_cert (ctrl->audit, AUDIT_ENCRYPTED_TO, cl->cert, err);
           log_error ("ksba_cms_add_recipient failed: %s\n",
                      gpg_strerror (err));
+          rc = err;
           xfree (encval);
           goto leave;
         }
@@ -820,6 +802,7 @@ gpgsm_encrypt (ctrl_t ctrl, certlist_t recplist, int data_fd, estream_t out_fp)
         {
           log_error ("ksba_cms_set_enc_val failed: %s\n",
                      gpg_strerror (err));
+          rc = err;
           goto leave;
         }
     }
@@ -827,15 +810,6 @@ gpgsm_encrypt (ctrl_t ctrl, certlist_t recplist, int data_fd, estream_t out_fp)
   if (compliant && gnupg_gcrypt_is_compliant (CO_DE_VS))
     gpgsm_status (ctrl, STATUS_ENCRYPTION_COMPLIANCE_MODE,
                   gnupg_status_compliance_flag (CO_DE_VS));
-  else if (opt.require_compliance
-           && opt.compliance == CO_DE_VS)
-    {
-      log_error (_("operation forced to fail due to"
-                   " unfulfilled compliance rules\n"));
-      gpgsm_errors_seen = 1;
-      err = gpg_error (GPG_ERR_FORBIDDEN);
-      goto leave;
-    }
 
   /* Main control loop for encryption. */
   recpno = 0;
@@ -844,7 +818,8 @@ gpgsm_encrypt (ctrl_t ctrl, certlist_t recplist, int data_fd, estream_t out_fp)
       err = ksba_cms_build (cms, &stopreason);
       if (err)
         {
-          log_error ("creating CMS object failed: %s\n", gpg_strerror (err));
+          log_debug ("ksba_cms_build failed: %s\n", gpg_strerror (err));
+          rc = err;
           goto leave;
         }
     }
@@ -853,15 +828,15 @@ gpgsm_encrypt (ctrl_t ctrl, certlist_t recplist, int data_fd, estream_t out_fp)
   if (encparm.readerror)
     {
       log_error ("error reading input: %s\n", strerror (encparm.readerror));
-      err = gpg_error (gpg_err_code_from_errno (encparm.readerror));
+      rc = gpg_error (gpg_err_code_from_errno (encparm.readerror));
       goto leave;
     }
 
 
-  err = gnupg_ksba_finish_writer (b64writer);
-  if (err)
+  rc = gnupg_ksba_finish_writer (b64writer);
+  if (rc)
     {
-      log_error ("write failed: %s\n", gpg_strerror (err));
+      log_error ("write failed: %s\n", gpg_strerror (rc));
       goto leave;
     }
   audit_log (ctrl->audit, AUDIT_ENCRYPTION_DONE);
@@ -876,5 +851,5 @@ gpgsm_encrypt (ctrl_t ctrl, certlist_t recplist, int data_fd, estream_t out_fp)
   xfree (dek);
   es_fclose (data_fp);
   xfree (encparm.buffer);
-  return err;
+  return rc;
 }
